@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2010-2016 Phusion Holding B.V.
+ *  Copyright (c) 2010-2017 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -35,6 +35,7 @@
 #include <pwd.h>
 #include <Constants.h>
 #include <Exceptions.h>
+#include <FileTools/FileManip.h>
 #include <Utils.h>
 #include <Utils/IniFile.h>
 
@@ -53,6 +54,7 @@ using namespace boost;
 class ResourceLocator {
 private:
 	string installSpec;
+	string packagingMethod;
 	string binDir;
 	string supportBinariesDir;
 	string helperScriptsDir;
@@ -61,6 +63,7 @@ private:
 	string rubyLibDir;
 	string nodeLibDir;
 	string buildSystemDir;
+	bool originallyPackaged;
 
 	static string getOption(const string &file, const IniFileSectionPtr &section, const string &key) {
 		if (section->hasKey(key)) {
@@ -86,7 +89,9 @@ public:
 	{
 		if (getFileType(_installSpec) == FT_REGULAR) {
 			const string &file = _installSpec;
+			originallyPackaged = false;
 			IniFileSectionPtr options = IniFile(file).section("locations");
+			packagingMethod     = getOption(file, options, "packaging_method");
 			binDir              = getOption(file, options, "bin_dir");
 			supportBinariesDir  = getOption(file, options, "support_binaries_dir");
 			helperScriptsDir    = getOption(file, options, "helper_scripts_dir");
@@ -97,6 +102,8 @@ public:
 			buildSystemDir      = getOptionalSection(file, options, "node_libdir");
 		} else {
 			const string &root = _installSpec;
+			originallyPackaged  = true;
+			packagingMethod     = "unknown";
 			binDir              = root + "/bin";
 			supportBinariesDir  = root + "/buildout/support-binaries";
 			helperScriptsDir    = root + "/src/helper-scripts";
@@ -108,10 +115,18 @@ public:
 		}
 	}
 
+	bool isOriginallyPackaged() const {
+		return originallyPackaged;
+	}
+
 	const string &getInstallSpec() const {
 		return installSpec;
 	}
 
+	const string &getPackagingMethod() const {
+		return packagingMethod;
+	}
+
 	const string &getBinDir() const {
 		return binDir;
 	}
